/*
 * Sistemas de Tempo-Real, 2003/2004
 * Paulo Pedreiras, Oct.2003
 *
 * Demonstracao do uso do Shark
 *
 * AULA 3 - aula3.c
 *
 * Application file
 *
 * Changelog
 * 2006/09/22, Paulo Pedreiras
 *   Updated to work with SHARK 1.5/1.51
 *   (include paths updated, sys_end/sys_abort obsolete,...)
 *   Changed task body to print task ID
 */


/**************************/
/*  Periodic task test    */
/**************************/
#define T0_Load  200000 /* Load in micro-seconds */
#define T1_Load  500000
#define T2_Load  500000
#define T3_Load 1000000

#include <ll/i386/cons.h>
#include <kernel/kern.h>
#include <drivers/shark_keyb26.h>

/* Some global task attributes */
#define TASK_WCET 1000

TASK utask0(void)
{
  unsigned char app_id;

  /* initialization within task0 */
  /* constant assignment of id */
  app_id=0;

  /* Print task ID label */
  putc_xy(10+app_id*5-1,9,YELLOW,'T');
  putc_xy(10+app_id*5+1,9,YELLOW,'0'+app_id);

  /* wait for next instance for regular execution */
  task_endcycle();

  /* Task body */
  while (1) {
    /* switch on character on right position */
    putc_xy(10+app_id*5,10,50,'x');

    /* wait on loop, simulating load */
    TIME t1;
    t1=sys_gettime(NULL);
    while(sys_gettime(NULL) < t1+T0_Load);

    /* switch off character */
    putc_xy(10+app_id*5,10,0,' ');

    /* Instance complete */
    task_endcycle();
  }
}


TASK utask1(void)
{
  unsigned char app_id;

  /* initialization within task0 */
  /* constant assignment of id */
  app_id=1;
        
  /* Print task ID label */
  putc_xy(10+app_id*5-1,9,YELLOW,'T');
  putc_xy(10+app_id*5+1,9,YELLOW,'0'+app_id);


  /* wait for next instance for regular execution */
  task_endcycle();

  /* Task body */
  while (1) {
    /* switch on character on right position */
    putc_xy(10+app_id*5,10,50,'x');

    /* wait on loop, simulating load */
    TIME t1;
    t1=sys_gettime(NULL);
    while(sys_gettime(NULL) < t1+T1_Load);

    /* switch on character on right position */
    putc_xy(10+app_id*5,10,0,' ');

    /* Instance complete */
    task_endcycle();
  }
}


TASK utask2(void)
{
  unsigned char app_id;

  /* initialization within task0 */
  /* constant assignment of id */
  app_id=2;

  /* Print task ID label */
  putc_xy(10+app_id*5-1,9,YELLOW,'T');
  putc_xy(10+app_id*5+1,9,YELLOW,'0'+app_id);

  /* wait for next instance for regular execution */
  task_endcycle();

  /* Task body */
  while (1) {
    /* switch on character on right position */
    putc_xy(10+app_id*5,10,50,'x');

    /* wait on loop, simulating load */
    TIME t1;
    t1=sys_gettime(NULL);
    while(sys_gettime(NULL) < t1+T2_Load);

    /* switch on character on right position */
    putc_xy(10+app_id*5,10,0,' ');

    /* Instance complete */
    task_endcycle();
  }
}


TASK utask3(void)
{
  unsigned char app_id;

  /* initialization within task0 */
  /* constant assignment of id */
  app_id=3;

  /* Print task ID label */
  putc_xy(10+app_id*5-1,9,YELLOW,'T');
  putc_xy(10+app_id*5+1,9,YELLOW,'0'+app_id);
        
  /* wait for next instance for regular execution */
  task_endcycle();

  /* Task body */
  while (1) {
    /* switch on character on right position */
    putc_xy(10+app_id*5,10,50,'x');

    /* wait on loop, simulating load */
    TIME t1;
    t1=sys_gettime(NULL);
    while(sys_gettime(NULL) < t1+T3_Load);

    /* switch on character on right position */
    putc_xy(10+app_id*5,10,0,' ');

    /* Instance complete */
    task_endcycle();
  }
}


/****************************************************************/
/* This function is called when Alt-X is pressed.
   It simply shutdown the system using exit.
*/
void my_end(KEY_EVT* e)
{
  exit(0);
}

/******************************************************************/

/* This function is called when the system exit correctly after Alt-X.
   It prints a small greeting.
   Note that:
   - The function is registered using RUNLEVEL_BEFORE_EXIT and
    so is executed in protected mode.
   - To execute some code after returning to real mode 
     (e.g. to use the DOS filesystem functions) it is the register 
     the functuon with RUNLEVEL_AFTER_EXIT
*/

void byebye(void *arg)
{
  kern_printf("Bye Bye!\n");
}



/****************************** MAIN ******************************/

int main(int argc, char **argv)
{

  KEY_EVT emerg;
  HARD_TASK_MODEL mh;
  PID pid[4];

  /* Set the closing function */
  sys_atrunlevel(byebye, NULL, RUNLEVEL_BEFORE_EXIT);


  //keyb_set_map(engMap);
  /* set the keyboard handler to exit correctly */
  emerg.ascii = 'x';
  emerg.scan = KEY_X;
  emerg.flag = ALTL_BIT;
  keyb_hook(emerg,my_end,FALSE);

  emerg.ascii = 'x';
  emerg.scan = KEY_X;
  emerg.flag = ALTR_BIT;
  keyb_hook(emerg,my_end,FALSE);

  /* a small banner */
  cprintf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
  cprintf("aula3 Demo\n");
  cprintf("Press Alt-X to exit\n");


  /* Task creation */

  /* Task's common attributes*/
  hard_task_default_model(mh);
  hard_task_def_wcet     (mh, TASK_WCET);

  /* Task's specific attributes */
  hard_task_def_arg      (mh, NULL);
  hard_task_def_mit      (mh, 1000000);
  hard_task_def_group    (mh, 2);

  pid[0] = task_create("utask0", utask0, &mh, NULL);
  if (pid[0] == NIL) {
    perror("Could not create task <utask0>");
    exit(2);
  }


  /* Task's specific attributes */
  hard_task_def_arg      (mh, NULL);
  hard_task_def_mit      (mh, 2000000);
  hard_task_def_group    (mh, 2);

  pid[1] = task_create("utask1", utask1, &mh, NULL);
  if (pid[1] == NIL) {
    perror("Could not create task <utask1>");
    exit(2);
  }



  /* Task's specific attributes */
  hard_task_def_arg      (mh, NULL);
  hard_task_def_mit      (mh, 3000000);
  hard_task_def_group    (mh, 2);

  pid[2] = task_create("utask2", utask2, &mh, NULL);
  if (pid[2] == NIL) {
    perror("Could not create task <utask2>");
    exit(2);
  }

  /* Task's specific attributes */
  hard_task_def_arg      (mh, NULL);
  hard_task_def_mit      (mh, 4000000);
  hard_task_def_group    (mh, 2);

  pid[3] = task_create("utask3", utask3, &mh, NULL);
  if (pid[3] == NIL) {
    perror("Could not create task <utask3>");
    exit(2);
  }

  group_activate(2);


  /*
    now the task main ends, but the system does not shutdown because
    there are tasks.The demo will finish if a Alt-X key is pressed.
  */

  return 0;
}

